/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
/*
** Copyright (C) 2000 Dirk-Jan C. Binnema <dirkjan@gnome.org>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/



#include <orbit/orbit.h>
#include <stdio.h>
#include <bonobo-activation/bonobo-activation.h>
#include "calculator.h"

static void
help_print (char *argv[])
{
	  printf ("syntax is: %s [option]\n", argv[0]);
	  printf ("where [option] is either --exe or --shlib\n");
	  printf ("\t--exe\t will activate an executable server\n");
	  printf ("\t--shlib\t will activate a shared library server\n");
}

int
main (int argc, char *argv[])
{
	CORBA_Environment ev;
	char *query, *new_query;
	calculator objref;

	CORBA_exception_init(&ev);
	bonobo_activation_init ( argc, argv );

	if (argc == 2) {
	  if (strcmp (argv[1], "--exe") == 0) {
	    query = "(server_type == 'exe')";
	  } else if (strcmp (argv[1], "--shlib") == 0) {
	    query = "(server_type == 'shlib')";
	  } else {
	    help_print (argv);
	    return 1;
	  }
	} else {
	    help_print (argv);
	  return 0;
	}

	new_query = g_strconcat (query, " AND ", " repo_ids.has('IDL:calculator:1.0')", NULL);
	objref = bonobo_activation_activate ( new_query, NULL, 0, NULL, &ev );
	if (CORBA_OBJECT_NIL == objref 
	    || ev._major != CORBA_NO_EXCEPTION) {
		printf ("Could not bind to server\n" );
		return 1;
	}

	/* do some test */	
	printf ( "3 + 4 = %d\n", calculator_add ( objref, 3, 4, &ev));
	if (ev._major != CORBA_NO_EXCEPTION) {
		printf ("Could not access object.\n" );
		return 1;
	}

	CORBA_Object_release( objref, &ev);
	CORBA_exception_free ( &ev );

	return 0;
}


