/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

#include <stdio.h>
#include <bonobo-activation/bonobo-activation.h>
#include "calculator.h"

extern calculator impl_create   (PortableServer_POA    poa, 
				 CORBA_Environment   * ev);

/* Here, the server has no idea when it can unload itself 
   from memory. It will thus stay there for quite some time.
   If the server knew when to destroy itself (bonobo servers 
   typically know this, it would need to call:
   bonobo_activation_plugin_unuse (poa, impl_ptr); and then 
   impl_calculator__destroy to destroy the relevant implementation

   The above would ensure that: 
       1) the implementation is destroyed.
       2) bonobo-activation unloads the shared library from memory.
*/


/* basic activation function for the library.
   This function is supposed to return a new CORBA object
   for the requested IID.
   It is called by bonobo-activation on behalf of the application.

   impl_ptr is a pointer to be stored by the implementation
   which it is supposed to pass to bonobo_activation_plugin_use to register 
   into the server
*/
static CORBA_Object
calculator_shlib_make_object (PortableServer_POA poa,
		      const char *iid,
		      gpointer impl_ptr,
		      CORBA_Environment *ev)
{
	CORBA_Object object_ref;

	/* FIXME: is this necessary ? */
	PortableServer_POAManager_activate (PortableServer_POA__get_the_POAManager (poa, ev), ev);

	object_ref = impl_create (poa, ev);
	if (object_ref == CORBA_OBJECT_NIL 
	    || ev->_major != CORBA_NO_EXCEPTION) {
		printf ("Server cannot get objref\n");
		return CORBA_OBJECT_NIL;
	}

	bonobo_activation_plugin_use (poa, impl_ptr);

	return object_ref;
}


/*
  This structure contains the list of plugins oaf can load from this library.
  Each plugin has an IID and a _factory_ function.
*/
static const BonoboActivationPluginObject calculator_plugin_list[] = {
	{
  		"OAFIID:calculator:20000923",
		calculator_shlib_make_object
	},
  	{
  		NULL
  	}
};


/* exported symbol which bonobo-activation tries to find to 
   bootstrap the library */
const BonoboActivationPlugin Bonobo_Plugin_info = {
	calculator_plugin_list,
	"Calculator example"
};






