#include "calculator.h"

/*** App-specific servant structures ***/

typedef struct
{
   POA_calculator servant;
   PortableServer_POA poa;

}
impl_POA_calculator;

/*** Implementation stub prototypes ***/

static void impl_calculator__destroy(impl_POA_calculator * servant,
				     CORBA_Environment * ev);
static CORBA_long
impl_calculator_add(impl_POA_calculator * servant,
		    const CORBA_long x,
		    const CORBA_long y, CORBA_Environment * ev);

static CORBA_long
impl_calculator_sum(impl_POA_calculator * servant,
		    const calculator_Numbers * numbers,
		    CORBA_Environment * ev);

static CORBA_double
impl_calculator_divide(impl_POA_calculator * servant,
		       const CORBA_long x,
		       const CORBA_long y, CORBA_Environment * ev);

/*** epv structures ***/

static PortableServer_ServantBase__epv impl_calculator_base_epv = {
   NULL,			/* _private data */
   NULL,			/* finalize routine */
   NULL,			/* default_POA routine */
};
static POA_calculator__epv impl_calculator_epv = {
   NULL,			/* _private */
   (gpointer) & impl_calculator_add,

   (gpointer) & impl_calculator_sum,

   (gpointer) & impl_calculator_divide,

};

/*** vepv structures ***/

static POA_calculator__vepv impl_calculator_vepv = {
   &impl_calculator_base_epv,
   &impl_calculator_epv,
};

/*** Stub implementations ***/

static calculator
impl_calculator__create(PortableServer_POA poa, CORBA_Environment * ev)
{
   calculator retval;
   impl_POA_calculator *newservant;
   PortableServer_ObjectId *objid;

   newservant = g_new0(impl_POA_calculator, 1);
   newservant->servant.vepv = &impl_calculator_vepv;
   newservant->poa = poa;
   POA_calculator__init((PortableServer_Servant) newservant, ev);
   objid = PortableServer_POA_activate_object(poa, newservant, ev);
   CORBA_free(objid);
   retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

   return retval;
}

static void
impl_calculator__destroy(impl_POA_calculator * servant,
			 CORBA_Environment * ev)
{
   PortableServer_ObjectId *objid;

   objid = PortableServer_POA_servant_to_id(servant->poa, servant, ev);
   PortableServer_POA_deactivate_object(servant->poa, objid, ev);
   CORBA_free(objid);

   POA_calculator__fini((PortableServer_Servant) servant, ev);
   g_free(servant);
}

static CORBA_long
impl_calculator_add(impl_POA_calculator * servant,
		    const CORBA_long x,
		    const CORBA_long y, CORBA_Environment * ev)
{
   CORBA_long retval;
   /* folowing line added */
   retval = x + y;
   return retval;
}

static CORBA_long
impl_calculator_sum(impl_POA_calculator * servant,
		    const calculator_Numbers * numbers,
		    CORBA_Environment * ev)
{
   CORBA_long retval;
   /* folowing lines added. */
   unsigned int i;
   retval = 0;
   for ( i = 0; i < numbers->_length; i++ ) {
     retval += numbers->_buffer[i];
   }

   return retval;
}

static CORBA_double
impl_calculator_divide(impl_POA_calculator * servant,
		       const CORBA_long x,
		       const CORBA_long y, CORBA_Environment * ev)
{
   CORBA_double retval;
   /*
    * raise an exception when dividing by zero
    */ 
   if ( 0 == y ) {
   
     retval = 0;
     CORBA_exception_set ( ev, CORBA_USER_EXCEPTION, ex_calculator_DivideByZero, NULL );
   } else {
     retval = x / y;
   }
   
   return retval;
}

/* folowing function added */
calculator impl_create (PortableServer_POA poa, CORBA_Environment * ev)
{
  return impl_calculator__create (poa, ev);
}
